<?php
App::uses('CustomEmail', 'Lib');

/**
 * Handles User requests
 *
 * @package CoreCMS/User
 */
class UsersController extends AppController {

/**
 * List of actions permitted by admin users.
 */
	public $adminActions = array(
		'admin_index',
		'admin_add',
		'admin_edit',
		'admin_account',
		'admin_delete',
		'admin_toggle',
		'admin_password_reset',
		'admin_password_reset_email',
		'admin_password_reset_callback',
		'admin_login'

	);


/**
 * Allow public access to some methods.
 */
	public function beforeFilter() {

		parent::beforeFilter();

		$this->Auth->allow(array(
			'admin_login',
			'admin_password_reset',
			'admin_password_reset_email',
			'admin_password_reset_callback',
			'password_reset',
			'password_reset_email',
			'password_reset_callback',
			'password_reset_fail'
		));

	}


/**
 * Handles User login. Redirects back to the calling page on successful login.
 */
	public function login() {

		$this->__login();
		return;

	}


/**
 * Admin login.
 */
	public function admin_login() {

		$this->__login();

		$this->layout = "admin_login";

		return;

	}


/**
 * Login - with login attempt time-out built in.
 */
	private function __login() {

		$this->User->validate = $this->User->validateLogin();

		if (!empty($this->request->data)) {

			$this->User->set($this->request->data);

			// Check if a delay is being applied to the login
			$loginDelay = $this->Session->read('Login.delay');

			if ($loginDelay && $loginDelay>time()) {

				$minutes = intval(($loginDelay - time()) / 60) + 1;

				$this->Session->setFlash(
					array(
						'title' => __('Login failed'),
						'description' => __('You will not be able to login for the next %s minute(s)!', $minutes)
					),
					'flash_fail'
				);

				return;

			}

			if ($this->User->validates()) {

				if ($this->Auth->login()) {

					// Unset the login attempt limits
					$this->Session->write('Login.attempt', 0);
					$this->Session->write('Login.delay', 0);

					//Get a better version of the user model out in the format $data['User']
					$user = $this->Auth->user();

					$fullUser = $this->User->getUserForLogin($user['id']);

					$this->Auth->login($fullUser);

					return $this->redirect($this->Auth->redirect());

				}
				else {

					// Define the time delays for limiting login attempts
					$loginDelays = array(0, 1, 5, 30, 60, 90);

					// Check login limits and write updated ones to session
					$loginAttempt = $this->Session->read('Login.attempt') ? $this->Session->read('Login.attempt') + 1 : 1;
					$loginDelay = isset($loginDelays[intval($loginAttempt/5)]) ? $loginDelays[intval($loginAttempt/5)] : 90;
					$this->Session->write('Login.attempt', $loginAttempt);
					$this->Session->write('Login.delay', time() + $loginDelay*60);

					if ($loginDelay>0) {

						$this->Session->setFlash(
							array(
								'title' => __('Login failed'),
								'description' => __('You will not be able to login for the next %s minute(s)!', $loginDelay)
							),
							'flash_fail'
						);

					} else {

						$this->Session->setFlash(__('Username or password is incorrect'), 'flash_fail');

					}

				}

			}

		}

	}


/**
 * Handles User logout. Redirects to Auth::logoutAction().
 */
	public function logout() {

		$this->Auth->logout();

		$this->redirect("/");
		return;

	}


/**
 * Password reset.
 */
	public function password_reset() {

		return $this->__password_reset();

	}


/**
 * password reset when in admin section
 */
	public function admin_password_reset() {

		$this->__password_reset();

		$this->layout = 'admin_login';

		return;

	}


/**
 * User password reset form.
 */
	private function __password_reset() {

		if (!empty($this->request->data)) {

			$user = $this->User->findByEmail($this->request->data['User']['email']);

			if (!$user) {

				$this->Session->setFlash('Email not found', 'flash_fail');

			}
			else {

				$this->redirect(array('controller' => 'users' , 'action' => 'password_reset_email', $user['User']['id']));

			}

		}

	}


	public function password_reset_email($userId) {

		$this->__password_reset_email($userId);

	}


	public function admin_password_reset_email($userId) {

		$this->__password_reset_email($userId);

		$this->layout = 'admin_login';

	}


/**
 * Creates a reset code and emails it to the user
 *
 * @param integer $userId ID of user to reset password for
 */
	private function __password_reset_email($userId) {

		$user = $this->User->read(null, $userId);

		if (!$user) {

			$this->Session->setFlash('User not found.', 'flash_fail');
			$this->redirect('/user_account/password_reset');

		}

		$code = $user['User']['password_reset_code'];

		if ($code == "") {

			$code = substr(md5(uniqid()), 0, 10);

			$this->User->saveField('password_reset_code', $code);

		}

		// send the email
		$email = new CustomEmail();

		$email->template('password_reset_email');
		$email->to($user['User']['email']);
		$email->from(array(Configure::read('SiteSetting.admin_email') => Configure::read('SiteSetting.site_title')));
		$email->subject(Configure::read('SiteSetting.site_title') . ' Password Reset');
		$email->viewVars(array(
			'user' => $user,
			'code' => $code,
			'url' => Router::url(
				array(
					'controller' => 'users',
					'action' => 'password_reset_callback',
					$code
				),
				true
			)
		));
		$email->helpers(array(
				'Html'
		));
		$email->send();

		$this->set('resetUser', $user);

		return;

	}


/**
 * Handles callback from password reset email. Allows a user to change their
 * password.
 *
 * @param string $code
 */
	public function password_reset_callback($code) {

		$this->User->validate = $this->User->validatePassword();


		$user = $this->User->findByPasswordResetCode($code);

		if (!$user) {

			$this->Session->setFlash('Password reset code not found', 'flash_fail');
			$this->redirect('/user_account/password_reset');

		}

		$this->__password_reset_callback($code, $user);

		return;
	}

	/**
	 * Handles callback from password reset email
	 * Allows user to change their password
	 *
	 * @param unknown_type $code
	 */
	public function admin_password_reset_callback($code) {

		$this->User->validate = $this->User->validatePassword();


		$user = $this->User->findByPasswordResetCode($code);

		if (!$user) {

			$this->Session->setFlash('Password reset code not found', 'flash_fail');
			$this->redirect('/admin/users/password_reset');
		}

		$this->__password_reset_callback($code, $user);

		$this->layout = 'admin_login';
	}

	/**
	 * Handles callback from password reset email
	 * Allows user to change their password
	 *
	 * @param unknown_type $code
	 * @param array user array from findByPasswordResetCode
	 */
	private function __password_reset_callback($code, $user)
	{
		if (!empty($this->request->data)) {

			$this->request->data['User']['id'] = $user['User']['id'];
			$this->request->data['User']['password_reset_code'] = "";

			if ($this->User->save($this->request->data)) {

				$user = $this->User->findByPasswordResetCode($code);
				$this->Session->setFlash('Password changed', 'flash_success');

				$this->redirect(array(
					'action' => 'login'
				));

			}
			else {

				$this->Session->setFlash('Please correct the errors:', 'flash_fail');

			}
		}
	}


/**
 * Populates drop down lists for admin forms.
 */
	protected function _adminPopulateLookups() {

		$Model = $this->{$this->modelClass};

		$this->set('userGroups', $Model->UserGroup->find('list'));

		return;

	}


/**
 * Defines the fields used in the admin filter form.
 *
 * @return array
 */
	protected function _adminFilterFields() {

		$Model = $this->{$this->modelClass};

		$this->_adminPopulateFilterLookups();

		$fields = parent::_adminFilterFields();

		$fields[$Model->alias.'.user_group_id']['type'] = "select";

		unset($fields[$Model->alias.'.password']);
		unset($fields[$Model->alias.'.password_reset_code']);
		unset($fields[$Model->alias.'.password_reset_code_expires']);

		return $fields;

	}


/**
 * Defines the columns to display for admin_index().
 *
 * @return array
 */
	protected function _adminIndexColumns() {

		$Model = $this->{$this->modelClass};
		
		return array(
			$Model->alias.'.id'=>array(
				'label'=>'ID',
				'type'=>'integer',
			),
			$Model->alias.'.email'=>array(
				'label'=>'Email',
				'type'=>'string',
			),
			'UserGroup.name'=>array(
				'label'=>'User Group',
				'type'=>'string'
			)
		);

	}

	protected function _adminIndexPaginate() {

		$paginate = parent::_adminIndexPaginate();
		$paginate['contain'][] = 'UserGroup';

		if ($this->Auth->user('User.id') != 1) {

			$paginate['conditions']['User.id !='] = 1;
		}

		return $paginate;
	}

	protected function _adminFormFields() {

		$fields = parent::_adminFormFields();

		$Model = $this->{$this->modelClass};

		$fields[$Model->alias.'.password']['div'] = 'input text required';
		$fields[$Model->alias.'.password']['type'] = 'password';

		unset(
			$fields[$Model->alias.'.password_reset_code'],
			$fields[$Model->alias.'.password_reset_code_expires']
		);

		return $fields;
	}

	/**
	 * Add or edit a user.
	 *
	 * @param  integer $id User ID
	 */
	public function admin_edit($id = null) {

		// unset password fields if they're not needed
		if (! empty($this->request->data)) {

			if (empty($this->request->data['User']['password'])) {

				unset($this->request->data['User']['password'], $this->request->data['User']['confirm_password']);
			}
		}

		// call the parent admin_edit to do the add / edit
		parent::admin_edit($id);

		// unset again incase it was prefilled
		$this->request->data['User']['password'] = "";
		$this->request->data['User']['confirm_password'] = "";
	}


/**
 * Admin 'My Account'.
 */
	public function admin_account() {

		$user = $this->Auth->user();
		$Model = $this->{$this->modelClass};

		$Model->validate = $Model->validateEdit();

		if (!empty($this->request->data)) {

			// Ensure the user is updating their own record only.
			$this->request->data['User']['id'] = $user['User']['id'];

			if ($this->User->saveAll($this->request->data)) {

				$this->Session->setFlash(array(
					'title' => "Account Updated",
					'description' => "Your account has been successfully updated!"
				), 'flash_success');
				return $this->redirect(array('action'=>'account'));

			} else {

				$this->Session->setFlash(array(
					'title' => 'Save failed',
					'description' => 'Failed to update your account'
				), 'flash_fail');

			}

		} else {

			$this->request->data = $this->User->readForEdit($user['User']['id']);

		}

		$this->_adminPopulateLookups();

		// unset again incase it was prefilled
		$this->request->data['User']['password'] = "";
		$this->request->data['User']['confirm_password'] = "";

		$this->set('title_for_layout', 'My Account');


	}


/**
 * Edit current user account.
 */
	public function account() {

		$Model = $this->{$this->modelClass};

		$user = $this->Auth->user();

		$Model->validate = $Model->validateEdit();

		if (!empty($this->request->data)) {

			$this->request->data['User']['id'] = $user['User']['id'];

			if ($this->request->data['User']['password'] == "") {

				unset($this->request->data['User']['password']);
				unset($this->request->data['User']['conform_password']);

			}
			else {

				$Model->validate = array_merge_recursive($Model->validate, $Model->validatePassword());

			}

			if ($this->User->save($this->request->data)) {

				$this->Session->setFlash('Information updated', 'flash_success');
				return $this->redirect('/users/account');

			}

		}
		else {

			$this->request->data = $this->User->findById($user['User']['id']);

		}

		$this->request->data['User']['password'] = "";
		$this->request->data['User']['confirm_password'] = "";

		$this->set('title_for_layout', 'Update Account Information | '.Configure::read('SiteSetting.site_title'));

		return;

	}

}
